import { tokenize as prismTokenize, languages } from './vendor/prism';
var newlineRe = /(\r\n|\r|\n)/; // Takes an array of Prism's tokens and groups them by line, turning plain
// strings into tokens as well. The length key is consciously kept precise
// so that the original string can be reliably reconstructed.

var groupTokens = function groupTokens(tokens) {
  var tokensSize = tokens.length;
  var line;
  var lines = [line = []];

  for (var i = 0; i < tokensSize; i++) {
    var token = tokens[i];

    if (typeof token !== 'string') {
      // Tokens are directly pushed onto the current line
      line.push(token);
    } else {
      // Plain strings are split by newlines
      var parts = token.split(newlineRe);
      var partsSize = parts.length; // For each part the current line must be ended and a new line must be started

      for (var j = 0; j < partsSize; j += 2) {
        var part = parts[j];
        var nextPart = parts[j + 1]; // Plain string tokens are marked using the "plain" token

        line.push({
          type: 'plain',
          content: part,
          length: part.length + (nextPart !== undefined ? nextPart.length : 0)
        }); // The last part doesn't require a new line

        if (nextPart !== undefined) {
          lines.push(line = []);
        }
      }
    }
  }

  return lines;
};

export var tokenize = function tokenize(code, language) {
  // Take grammar from Prism's languages and call its "tokenize" fn
  var grammar = languages[language];
  var tokens = prismTokenize(code, grammar, language);
  return groupTokens(tokens);
};